// -*-c++-*-
//===========================================================
//=     University of Illinois at Urbana/Champaign          =
//=     Department of Computer Science                      =
//=     Dr. Dan Roth - Cognitive Computation Group          =
//=                                                         =
//=  Project: Snow                                          =
//=                                                         =
//=   Module: Network.h                                     =
//=  Version: 2.1                                           =
//=  Authors: Jeff Rosen, Andrew Carlson                    =
//=     Date: xx/xx/99                                      = 
//=                                                         =
//= Comments:                                               =
//===========================================================

#ifndef __NETWORK_H__
#define __NETWORK_H__

#include "Target.h"
#include "TargetIdSet.h"
#include "TargetRank.h"
#include "Cloud.h"
#include <vector>
#include <numeric>
#include <iomanip>

using namespace std;

typedef vector<LearningAlgorithm*> AlgorithmVector;

class Network
{
public:
    Network();
    Network( TargetIdSet& labs, LearningAlgorithm* pAlg );
    Network( const Network& net );
  
    Network& operator=( const Network& rhs );
  
    bool operator==( const Network& rhs ) const;
    bool operator!=( const Network& rhs ) const;
    bool operator<( const Network& rhs ) const;
    bool operator<=( const Network& rhs ) const;
    bool operator>( const Network& rhs ) const;
    bool operator>=( const Network& rhs ) const;
  
    // Network construction from specification
    bool CreateStructure( const string& spec );
    void CreateClouds( TargetIdSet& tars, AlgorithmVector& algorithms );     
  
    // Manual cloud construction
    void AddCloud( const Cloud& newcloud );
  
    // Functions used during training
    bool PresentExample( Example& ex );
    void TrainingComplete();    
    void NormalizeConfidence();
    void Discard();

    // Functions used during evaluation and on-line learning
    void RankTargets( Example& ex, TargetRanking& ranking );
    void ResetCounters();
    void ShowStatistics( Counter total );
  
    // Output the network to the console
    void Show();
    void WriteAlgorithms( ofstream& out );
    
    // Network persistence
    void Read( ifstream& in );
    void Write( ofstream& out );
  
protected:
    typedef vector<Cloud> CloudVector;
  
    CloudVector clouds;
    AlgorithmVector algorithms;
};

inline Network::Network()
{
}

inline void Network::AddCloud( const Cloud& newcloud )
{
    clouds.push_back(newcloud);
}

inline void Network::NormalizeConfidence()
{
    CloudVector::iterator it = clouds.begin();
    CloudVector::iterator end = clouds.end();
    for ( ; it != end; it++)
    {
        it->NormalizeConfidence();
    }
}


inline void Network::ResetCounters()
{
    CloudVector::iterator it = clouds.begin();
    CloudVector::iterator end = clouds.end();
    for ( ; it != end; it++)
    {
        it->ResetCounters();
    }
}

inline void Network::ShowStatistics( Counter total )
{
    // Show each target...
    CloudVector::iterator it = clouds.begin();
    CloudVector::iterator end = clouds.end();
    for ( ; it != end; it++)
    {
        it->ShowStatistics(total);
    }
}

inline void Network::WriteAlgorithms( ofstream& out )
{
    AlgorithmVector::iterator it = algorithms.begin();
    AlgorithmVector::iterator end = algorithms.end();
    int i;
  
    for (i = 1; it != end; it++, i++)
    {
        out << i 
            << ":  ";
        (*it)->Show(out);
    }
}

#endif
